import { globalSetting } from '../global';

import xOffsetUpdate from './x-offset-update';
import yOffsetUpdate from './y-offset-update';

import xScaleUpdate from './x-scale-update';


const xScaleLevels = {
    1: 10 / 9,
    2: 10 / 7,
    3: 10 / 6,
    4: 10 / 5,
    5: 10 / 4
};

// TODO: 优化横向滚动体验
const xOffsetLevel = {
    1: 500,
    2: 20,
    3: 5,
    4: 1,
    5: 0.5
};

const yOffsetLevel = {
    1: 20,
    2: 40,
    3: 80,
    4: 160,
    5: 320
};

const pluso = {
    desc: 'Zoom in time',
    fn: pstate => xScaleUpdate(pstate, () => pstate.xScale * xScaleLevels[globalSetting.HorizontalScalingRatio])
};

const minuso = {
    desc: 'Zoom out time',
    fn: pstate => xScaleUpdate(pstate, () => pstate.xScale / xScaleLevels[globalSetting.HorizontalScalingRatio])
};

const fullo = {
    desc: 'All of time',
    fn: pstate => xScaleUpdate(pstate, () => pstate.xScaleMin)
};

const scroll = {
    left: {
        desc: 'Scroll into the past',
        fn: pstate => xOffsetUpdate(pstate, () => pstate.xOffset + pstate.time / xOffsetLevel[globalSetting.HorizontalRollRatio])
    },
    right: {
        desc: 'Scroll into the future',
        fn: pstate => xOffsetUpdate(pstate, () => pstate.xOffset - pstate.time / xOffsetLevel[globalSetting.HorizontalRollRatio])
    },
    up: {
        desc: 'scroll up',
        fn: pstate => yOffsetUpdate(pstate, () => pstate.yOffset - yOffsetLevel[globalSetting.VerticalRollRario])
    },
    down: {
        desc: 'scroll down',
        fn: pstate => yOffsetUpdate(pstate, () => pstate.yOffset + yOffsetLevel[globalSetting.VerticalRollRario])
    }
};

const editable = {
    desc: 'Toggle edit mode',
    fn: (pstate, cm) => {
        console.log('editable', pstate, cm);
    }
};

export default {
    // Alt +  <,  >.    left / right
    // 'Alt+,': scroll.left,
    // 'Alt+.': scroll.right,

    // // Alt +  [  ]      home / end
    // 'Alt+[': { desc: 'Jump to beginning of time', fn: pstate => xOffsetUpdate(pstate, pstate.sidebarWidth) }, // Home

    // 'Alt+]': { desc: 'Jump to end time', fn: pstate => xOffsetUpdate(pstate, pstate.width - pstate.xScale * pstate.time) }, // End

    // // ALT +  -  +
    // 'Alt+=': pluso,  // '+': pluso,  '=': pluso,
    // 'Alt+-': minuso, // '-': minuso, '_': minuso,
    // 'Alt+0': fullo, // 'Shift+f': fullo, F: fullo, 'Shift+F': fullo,

    // wheel
    scrollLeft: scroll.left,
    scrollRight: scroll.right,
    scrollUp: scroll.up,
    scrollDown: scroll.down,

    'Shift+icon:scrollUp': scroll.left,
    'Shift+icon:scrollDown': scroll.right,
    'Ctrl+icon:scrollUp': pluso,
    'Ctrl+icon:scrollDown': minuso,


    nop: { fn: () => false }
};
